/*
    Keystone version 1.0 -- A group key management service
    Copyright (C) 2000  The University of Texas at Austin.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
    USA.

    Keystone version 1.0 was written by Chung Kei Wong for a research project
    directed by Professor Simon S. Lam, Department of Computer Sciences.  
 */

/*
    For a description of the techniques implemented herein as well as a 
    performance analysis please see the article "Secure Group Communications
    Using Key Graphs," IEEE/ACM Transactions on Networking, 8(1):16-30,
    February 2000. (An early version in Proceedings ACM SIGCOMM '98.)
    Also see the article "Keystone: A Group Key Management Service,"
    in Proceedings of International Conference on Telecommunications
    ICT 2000, May 2000.

    Submit questions to:

	Dr. Chung Kei Wong                  Email:  ckwong@alumni.utexas.net
	Inktomi Corporation
	Foster City, CA 94080 USA

        or

        Professor Simon S. Lam              Email:  lam@cs.utexas.edu
        Dept. of Computer Sciences          Phone:  (512) 471-9531
        The University of Texas at Austin   Fax:    (512) 471-8885
        Austin, Texas 78712 USA

 */

/* interface specific to CryptoLib */
#include "libcrypt.h"

#ifndef commontype
#define commontype
typedef	unsigned char	u8;
typedef unsigned short	u16;
typedef unsigned long	u32;
typedef	signed char	s8;
typedef signed short	s16;
typedef signed long	s32;
#endif

/****************************************************************************
 * constant definitions
 ****************************************************************************/

/* cryptographic functions */

/* encryption algorithms/key types */
#define ENC_DES_CBC	1
#define ENC_DES_ECB	2
#define ENC_DES3_CBC	3
#define ENC_DES3_ECB	4

/* signature schemes */
#define SIG_RSA		16
#define SIG_DSA		17

/* message digest functions */
#define MAC_MD5		32
#define MAC_SHS		33


/****************************************************************************
 * message structure
 ****************************************************************************/

#define MAX_MSGSIZE	4096

typedef struct MsgRec_ {
    unsigned char *msg;		/* message */
    int size;			/* current msg size */
    int max;			/* max msg size */
} MsgRec;


/****************************************************************************
 * key structure
 ****************************************************************************/

typedef struct Key_ {
    int keyType;	/* key type */
    int keyID;		/* key ID number */
    int version;	/* key version */
    void *key;		/* pointer to DESKey, DES3Key, or RSAKey */
} Key;

/* key sizes in bytes */
#define DES_KEY_SIZE		8
#define DES3_KEY_SIZE		24
#define MD5_SIZE		16
#define SHS_SIZE		20
#define MAX_MACSIZE		20
#define MAX_SIGSIZE		1024

typedef struct DESKey_ {
    unsigned char key[DES_KEY_SIZE];
    unsigned char iv[DES_KEY_SIZE];
    int mode;			/* encryption mode */
} DESKey;

typedef struct DES3Key_ {
    unsigned char store[DES3_KEY_SIZE];
    unsigned char *key[3];
    unsigned char iv[DES_KEY_SIZE];
    int mode;                   /* encryption mode */
} DES3Key;

typedef struct RSAKey_ {
    RSAKeySet *key;
    int modSize;		/* modulus size in bits */
} RSAKey;

typedef struct DSAKey_ {
    DSAKeySet *key;
    int modSize;		/* modulus size in bits */
} DSAKey;


/****************************************************************************
 * SA structure
 ****************************************************************************/

/* a special group SA includes rekeying addr and port, signing key, ... */
#define SERVER_SA_SPI		1	/* server SA spi */

/* security association SA */
typedef struct SA_ {
    unsigned long spi;		/* SPI */
    unsigned long ver;		/* version */
    unsigned long status;	/* status */
    /* encKey is circular array of keys in client */
    int nKeys;			/* num of keys in encKey */
    int currVerIndex;		/* index of current version */
    u32 exist;			/* bit vector for existence of key versions */
    Key *encKey;		/* encryption key */
    /* verification */
    int msgDigest;		/* msg digest */
    int chainType;		/* chaining type */
    Key *sigKey;		/* signing key */
} SA;

typedef struct SALst_ {
    SA *sa;
    struct SALst_ *next;
} SALst;

/****************************************************************************
 * function prototypes
 ****************************************************************************/

/* encrypt.c */
Key *newKey(int keyType, int par);
Key *newNKey(int n, int keyType, int par);
int freeKey(Key *key);
int genKey(Key *key);
int genDESkey(DESKey *key);
int genDES3key(DES3Key *key);
int genRSAkey(RSAKey *key);
int genDSAkey(DSAKey *key);
int fgetKey(Key *key, FILE *f);
int fgetRSAKey(RSAKey *key, FILE *f);
int fgetDSAKey(DSAKey *key, FILE *f);
int mputKey(Key *key, unsigned char *buf);
int mgetKey(Key *key, unsigned char *buf);
int bufPutDSAPublicKey_notable(DSAPublicKey *key, unsigned char **buffer);
DSAPublicKey *bufGetDSAPublicKey_notable(unsigned char **buffer);
int mputPublicKey(Key *key, unsigned char *buf);
int mgetPublicKey(Key *key, unsigned char *buf);
int encryptMsg(Key *key, MsgRec *msg);
int decryptMsg(Key *key, MsgRec *msg);
int encDES(DESKey *key, MsgRec *msg);
int decDES(DESKey *key, MsgRec *msg);
int encDES3(DES3Key *key, MsgRec *msg);
int decDES3(DES3Key *key, MsgRec *msg);
int signMsg(Key *key, MsgRec *msg, MsgRec *sig);
int verifyMsg(Key *key, MsgRec *msg, MsgRec *sig);
int signRSA(RSAKey *key, MsgRec *msg, MsgRec *sig);
int verifyRSA(RSAKey *key, MsgRec *msg, MsgRec *sig);
int signDSA(DSAKey *key, MsgRec *msg, MsgRec *sig);
int verifyDSA(DSAKey *key, MsgRec *msg, MsgRec *sig);
int genMAC(int micAlg, MsgRec *msg, MsgRec *mic);
int checkMAC(int micAlg, MsgRec *msg, MsgRec *mic);
int genMD5(MsgRec *msg, MsgRec *mic);
int checkMD5(MsgRec *msg, MsgRec *mic);
int genSHS(MsgRec *msg, MsgRec *mic);
int checkSHS(MsgRec *msg, MsgRec *mic);
int checkAuth(SA *sa, MsgRec *micMsg, MsgRec *authMsg);
SA *newSA(unsigned long spi);
int freeSA(SA *sa);
Key *getKeyVer(SA *sa, int ver);
u8 *removeAuthPay(MsgRec *msg, MsgRec *micMsg, MsgRec *authMsg);
int signMsgSA(MsgRec *msg, SA *sa);
int encMsgSA(MsgRec *msg, SA *sa);
int signEncMsgSA(MsgRec *msg, SA *sa);
int verifyMsgSA(MsgRec *msg, SA *sa);
int decMsgSA(MsgRec *msg, SA *sa);
int decVerifyMsgSA(MsgRec *msg, SA *sa);
int appendSAEnc1(MsgRec *msg, SA *sa, SA *encSA, u8 flags);
int appendSAEnc2(MsgRec *msg, SA *sa, u8 flags);
int appendSAFull2(MsgRec *msg, SA *sa, u8 flags);
int appendDeletedSA(MsgRec *msg, SA *sa);
int consNewSA2(SA *sa, u8 *saPay);
