/*
    Keystone version 1.0 -- A group key management service
    Copyright (C) 2000  The University of Texas at Austin.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
    USA.

    Keystone version 1.0 was written by Chung Kei Wong for a research project
    directed by Professor Simon S. Lam, Department of Computer Sciences.  
 */

/*
    For a description of the techniques implemented herein as well as a 
    performance analysis please see the article "Secure Group Communications
    Using Key Graphs," IEEE/ACM Transactions on Networking, 8(1):16-30,
    February 2000. (An early version in Proceedings ACM SIGCOMM '98.)
    Also see the article "Keystone: A Group Key Management Service,"
    in Proceedings of International Conference on Telecommunications
    ICT 2000, May 2000.

    Submit questions to:

	Dr. Chung Kei Wong                  Email:  ckwong@alumni.utexas.net
	Inktomi Corporation
	Foster City, CA 94080 USA

        or

        Professor Simon S. Lam              Email:  lam@cs.utexas.edu
        Dept. of Computer Sciences          Phone:  (512) 471-9531
        The University of Texas at Austin   Fax:    (512) 471-8885
        Austin, Texas 78712 USA

 */

#include "encrypt.h"
#include "sndrekey.h" /* need AddrRec */

#define MAX_GRPNAME	32
#define ADDR_MAX	40
#define MAX_GROUPS	32
/* number of previous versions of a key to be kept */
#define NUM_KEY_VERS	4

/* status of a group */
#define JOINED		0x01
#define JOINING		0x02
#define LEAVING		0x04

typedef struct AddrRec_ {
    char addr[ADDR_MAX];		/* ip address */
    int port, sk;			/* port and socket */
    struct sockaddr_in skAddr;
} AddrRec;

/* secure group record (client-side) */
typedef struct GroupRec_ {
    char name[MAX_GRPNAME];		/* group name (a string) */
    u32 id, supergrps, subgrps;		/* group id, supergroups, subgrps */
    AddrRec rekeyAddr;			/* rekey address */
    RekeyCtx *rekeyCtx;			/* rekey context */
    SA *grpSA;				/* group SA (group key) */
    u8 status;				/* JOINED, JOINING, LEAVING */
    int encryption;			/* encryption algorithm */
} GroupRec;

/* keystone context */
/* assume same encryption, signature, md for all groups in a server */
typedef struct KSCtx_ {
    /* request and rekey addresses */
    AddrRec reqAddr, rekeyAddr;
    int rekeying;			/* 1 if serv rekey addr for all grps */
    int deliveryType;			/* rekey delivery type */
    int sk;
    RekeyCtx *tcpMsgCtx;		/* for reply and rekey if TCP_UCAST */
    RekeyCtx *rekeyCtx;			/* rekey ctx may point to tcpCtx */
    u32 userID;				/* user id (given by this server) */
    SA *indSA;				/* individual SA */
    GroupRec *grp[MAX_GROUPS];		/* groups */
    int numGrps, maxGrps;
    int encryption;			/* encryption algorithm */
    int msgDigest;			/* message digest function */
    int signature;			/* signature scheme */
    SA * serverSA;			/* server sa (signature, md, key) */
    SALst * saLst;			/* sa's for groups with this server */
} KSCtx;

/****************************************************************************
 * function prototypes
 ****************************************************************************/

/* keyclient.c */
int ksCtxInit(KSCtx *ksCtx, char *file);
KSCtx *newKSCtx(char *file);
int fgetKSCtx(KSCtx *ksCtx, FILE *f);
int setupReqAddr(AddrRec *reqAddr);
int setupRekeyAddr(AddrRec *rekeyAddr);
GroupRec *searchGrp(KSCtx *ksCtx, char *grpName);
SA *findSA(KSCtx *ksCtx, u32 spi);
int deleteSA(KSCtx *ksCtx, u32 spi);
int keyUpdate(SA *sa, MsgRec *attrMsg, u32 ver);
int grpSAUpdate(GroupRec *grp, MsgRec *attrMsg, KSCtx *ksCtx);
int serverSAUpdate(KSCtx *ksCtx, MsgRec *attrMsg);
int procServerSA(KSCtx *ksCtx, u8 *curr);
int procGrpSA(GroupRec *grp, u8 *curr, KSCtx *ksCtx);
int procSA(KSCtx *ksCtx, u8 *curr);
int procRekey(MsgRec *msg, KSCtx *ksCtx, int firstTime);
int procAck(MsgRec *msg, KSCtx *ksCtx, u32 mid);
int reqGroups(KSCtx *ksCtx, int numGrps, char *grpName[], int op[]);
Key *getGroupKey(KSCtx *ksCtx, char *grpName, int ver);
SA *getGroupSA(KSCtx *ksCtx, char *grpName);
int printGroupSA(KSCtx *ksCtx, char *grpName);
