/*
    Keystone version 1.0 -- A group key management service
    Copyright (C) 2000  The University of Texas at Austin.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
    USA.

    Keystone version 1.0 was written by Chung Kei Wong for a research project
    directed by Professor Simon S. Lam, Department of Computer Sciences.  
 */

/*
    For a description of the techniques implemented herein as well as a 
    performance analysis please see the article "Secure Group Communications
    Using Key Graphs," IEEE/ACM Transactions on Networking, 8(1):16-30,
    February 2000. (An early version in Proceedings ACM SIGCOMM '98.)
    Also see the article "Keystone: A Group Key Management Service,"
    in Proceedings of International Conference on Telecommunications
    ICT 2000, May 2000.

    Submit questions to:

	Dr. Chung Kei Wong                  Email:  ckwong@alumni.utexas.net
	Inktomi Corporation
	Foster City, CA 94080 USA

        or

        Professor Simon S. Lam              Email:  lam@cs.utexas.edu
        Dept. of Computer Sciences          Phone:  (512) 471-9531
        The University of Texas at Austin   Fax:    (512) 471-8885
        Austin, Texas 78712 USA

 */

#include "encrypt.h"

#ifndef commontype
#define commontype
typedef	unsigned char	u8;
typedef unsigned short	u16;
typedef unsigned long	u32;
typedef	signed char	s8;
typedef signed short	s16;
typedef signed long	s32;
#endif

#define isLeaf(kn)      (((kn)->children == NULL) && ((kn)->nUsers == 1))

/* condition full(root,d) is TRUE iff tree/subtree rooted at root is full */
/* given d is the degree of the tree */
#define full(root,d) (intpow((d),((root)->height)) <= (root)->nUsers)

/* kn1 is smaller than kn2 if and only if */
/* (1) nChildren of kn1 < kn2 OR */
/* (2) (nChildren of kn1 = kn2 AND nUsers of kn1 < kn2) */
#define smaller1(kn1,kn2) (((kn1)->nChildren < (kn2)->nChildren) || \
    (((kn1)->nChildren == (kn2)->nChildren) \
	&& ((kn1)->nUsers < (kn2)->nUsers)))
#define smaller(kn1,kn2)	smaller1(kn1,kn2)

/* k-node and service status */
#define FIXED		0x01		/* this knode is fixed */
#define J_CHANGING	0x02		/* keys are j-changing */
#define L_CHANGING	0x04		/* keys are l-changing */
#define V_CHANGING	0x08		/* descendant keys are changing */
#define CHANGING	0x0E		/* keys are changing */
#define REKEYED		0x10		/* rekey msg sent */
#define NEW		0x20		/* this knode is new */
#define DUMMY_NODE	0x40		/* this is a dummy knode (not used) */

typedef struct KNode_ {
    int status;				/* status of this knode */
    u32 groupID;			/* group id if it's a group key */
    SA *currSA, *prevSA;		/* current sa and previous sa */
    u32 firstUsed;		/* time the curr encryption key first used */
				/* -1 if this SA is not used */
    /* parameters */
    int maxd;				/* max number of children (degree) */
    int height;				/* height of this node */
    int nChildren;			/* number of children */
    int nUsers;				/* number of users(leaves) */
    int avail;				/* dist to the closest avail des */
    /* parents and children */
    /* children are sorted according to function smaller() */
    /* schildren are FIXED nodes, eg, roots of JL-trees */
    struct KNodeLst_ * parent;		/* parent */
    struct KNodeLst_ * children;	/* doubly-linked list of children */
    struct KNodeLst_ * schildren;	/* doubly-linked list of schildren */
    /* used for rekey msgs */
    struct KNodeLst_ * deletedAnc;	/* deleted ancestors */
    /* for tcp rekey delivery */
    struct User_ *user;
} KNode;

typedef struct KNodeLst_ {
    KNode *kNode;			/* point to t_node */
    struct KNodeLst_ *next;		/* next */
    struct KNodeLst_ *prev;		/* previous */
} KNodeLst;

typedef struct KTree_ {
    KNode *root;
    int degree;
    int joining;
} KTree;

/* use to determine if a group key needs to be changed */
typedef struct GrpsChange_ {
    u32 grpsBefore;	/* groups before request */
    u32 grpsLeave;	/* groups requested to leave */
    u32 grpsJoin;	/* groups requested to join */
} GrpsChange;


/****************************************************************************
 * function prototypes
 ****************************************************************************/

/* keygraph.c */
unsigned long intpow(unsigned int a, unsigned int b);
int needChange(u32 group, GrpsChange *grpsCh);
KNode *newKNode(void);
KNode *newIntKNode(KTree *tree);
int freeKNode(KNode *kn);
int freeKNodeLst(KNodeLst *knl);
int freeKNodes(KNodeLst *knl);
KNodeLst *insertKNode(KNode *kn, KNodeLst *knl);
KNodeLst *removeKNode(KNode *kn, KNodeLst *knl);
KNodeLst *findKNode(KNode *kn, KNodeLst *knl);
KNodeLst *replaceKNode(KNode *kn1, KNode *kn2, KNodeLst *knl);
int orderSib(KNode *kn);
int orderAnc(KNode *kn);
int addOneUserAnc(KNode *kn);
int addnUserAnc(KNode *kn, int n);
int minusOneUserAnc(KNode *kn);
int updateHeightAnc(KNode *kn);
int updateAvailAnc(KNode *kn);
int removeChild(KNode *kn);
int addLeaf(KNode *leaf, KNode *parent);
int markChangeAnc(KNode *kn, int changing, GrpsChange *grpsCh, KNodeLst *changeNodes);
KNode *bestJoinPoint1(KNode *root);
KNode *bestJoinPoint2(KNode *root);
KNode *joinPoint(KTree *tree, int joining);
int join(KTree *tree, KNode *kn, GrpsChange *grpsCh, KNodeLst *changeNodes);
int leave(KTree *tree, KNode *kn, GrpsChange *grpsCh, KNodeLst *changeNodes);
