/*
    Keystone version 1.0 -- A group key management service
    Copyright (C) 2000  The University of Texas at Austin.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
    USA.

    Keystone version 1.0 was written by Chung Kei Wong for a research project
    directed by Professor Simon S. Lam, Department of Computer Sciences.  
 */

/*
    For a description of the techniques implemented herein as well as a 
    performance analysis please see the article "Secure Group Communications
    Using Key Graphs," IEEE/ACM Transactions on Networking, 8(1):16-30,
    February 2000. (An early version in Proceedings ACM SIGCOMM '98.)
    Also see the article "Keystone: A Group Key Management Service,"
    in Proceedings of International Conference on Telecommunications
    ICT 2000, May 2000.

    Submit questions to:

	Dr. Chung Kei Wong                  Email:  ckwong@alumni.utexas.net
	Inktomi Corporation
	Foster City, CA 94080 USA

        or

        Professor Simon S. Lam              Email:  lam@cs.utexas.edu
        Dept. of Computer Sciences          Phone:  (512) 471-9531
        The University of Texas at Austin   Fax:    (512) 471-8885
        Austin, Texas 78712 USA

 */

#ifndef commontype
#define commontype
typedef unsigned char	u8;
typedef unsigned short	u16;
typedef unsigned long	u32;
typedef signed char	s8;
typedef signed short	s16;
typedef signed long	s32;
#endif


/*************************************************************************
 * message
 *************************************************************************/

/* various header sizes (octet) */
#define MSG_HDR_SIZE		16
#define PAY_HDR_SIZE		4
#define KEY1_HDR_SIZE		20	/* group- and key-oriented rekey */
#define KEY2_HDR_SIZE		12	/* user-oriented rekey */

/* message type */
#define REQUEST		1	/* request */
#define ACK		2	/* acknowledgement */
#define NAK		3	/* negative acknowledgement */
#define REKEY1		4	/* graph/group/key-oriented rekey msg */
#define REKEY2		5	/* user-oriented rekey msg */
#define ACK_KEEP	6	/* keep (registration) tcp alive */

#define QUIT		10	/* register command: quit */
#define READY		11	/* ready for client registration */
#define REGISTER	12	/* register command: user registration */
#define UNREGISTER	13	/* register commend: unregister a user */
#define INIT_REGISTER	14	/* initialize register */

/* SA payload status field */
#define NEW_KEY		0x01
#define CHANGE_KEY	0x02
#define DELETE_KEY	0x04
#define SA_KEY		0x08

/* request/reply types (in request/reply payload) */
#define GRANTED		0x01
#define DENIED		0x02
#define JOIN_GRP		0x04
#define LEAVE_GRP		0x08
#define RESYN_GRP		0x10
#define JOIN_GRANTED	(JOIN_GRP|GRANTED)
#define JOIN_DENIED	(JOIN_GRP|DENIED)
#define LEAVE_GRANTED	(LEAVE_GRP|GRANTED)
#define LEAVE_DENIED	(LEAVE_GRP|DENIED)
#define RESYN_GRANTED	(RESYN_GRP|GRANTED)
#define RESYN_DENIED	(RESYN_GRP|DENIED)

/* payload type */
#define REQUEST_PAY	1
#define REPLY_PAY	2
#define CERT_PAY	3
#define KEY1_PAY	4
#define KEY2_PAY	5
#define MAC_PAY		6
#define SIG_PAY		7

/* SA attribute identifiers */
#define SPI		1
#define ENC_ALG		2
#define SIG_ALG		3
#define MD_ALG		4
#define VER_KEY		5
#define ENC_KEY		6

#define REKEY_DELIVERY	7
#define REKEY_ADDR	8
#define REKEY_PORT	9


/* round up to a multiple of 4 */              
#define up4(i)	((((i)+3) | 3) ^ 3)

#define payloadType(curr)		*(curr)
#define payloadSize(curr)		*((u16 *) (curr+2))
#define msgType(curr)			*(curr+1)

#define putMsgSize(curr,size)		\
	*((u16 *) (curr+2)) = htons(size);
#define getMsgSize(curr,size)		\
	*(size) = ntohs(*((u16 *) (curr+2)));
#define putPaySize(curr,size)		\
	*((u16 *) (curr+2)) = htons(size);
#define getPaySize(curr,size)		\
	*(size) = ntohs(*((u16 *) (curr+2)));
#define putPayHdr(curr,type,other,size)	\
	*(curr) = type; *(curr+1)=other; *((u16 *) (curr+2)) = htons(size);


/****************************************************************************
 * function prototypes
 ****************************************************************************/

/* msg.c */
int putMsgHdr(u8 *curr, u8 ver, u8 type, u16 size, u32 seq, u32 msgSPI, u32 msgVer);
int getMsgHdr(u8 *curr, u8 *ver, u8 *type, u16 *size, u32 *seq, u32 *msgSPI, u32 *msgVer);
int putMsgHdrSA(u8 *curr, u32 msgSPI, u32 msgVer);
int getMsgHdrSA(u8 *curr, u32 *msgSPI, u32 *msgVer);
int putRequestPay(u8 *curr, char *name, u8 reqType);
int getRequestPay(u8 *curr, char **name, u8 *reqType);
int putReplyPay(u8 *curr, char *name, u8 result);
int getReplyPay(u8 *curr, char **name, u8 *result);
int putMacPay(u8 *curr, u8 *mic, u16 micSize);
int getMacPay(u8 *curr, u8 **mic, u16 *micSize);
int putSigPay(u8 *curr, u8 *auth, u16 authSize);
int getSigPay(u8 *curr, u8 **auth, u16 *authSize);
int putKey1Pay(u8 *curr, u8 status, u16 size, u32 encSPI, u32 encVer, u32 contSPI, u32 contVer);
int getKey1Pay(u8 *curr, u8 *status, u16 *size, u32 *encSPI, u32 *encVer, u32 *contSPI, u32 *contVer);
int putKey2Pay(u8 *curr, u8 status, u16 size, u32 contSPI, u32 contVer);
int getKey2Pay(u8 *curr, u8 *status, u16 *size, u32 *contSPI, u32 *contVer);
int putAttrAV(u8 *curr, u16 attrType, u16 val);
int putAttrALV(u8 *curr, u16 attrType, u16 attrLen, u8 *val);
int getAttr(u8 *curr, u16 *attrType, u16 *ival, u8 **sval);
int putCertPay(u8 *curr, u8 *cert, int certSize);
int getCertPay(u8 *curr, u8 **cert, int *certSize);
