/*
    Keystone version 1.0 -- A group key management service
    Copyright (C) 2000  The University of Texas at Austin.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
    USA.

    Keystone version 1.0 was written by Chung Kei Wong for a research project
    directed by Professor Simon S. Lam, Department of Computer Sciences.  
 */

/*
    For a description of the techniques implemented herein as well as a 
    performance analysis please see the article "Secure Group Communications
    Using Key Graphs," IEEE/ACM Transactions on Networking, 8(1):16-30,
    February 2000. (An early version in Proceedings ACM SIGCOMM '98.)
    Also see the article "Keystone: A Group Key Management Service,"
    in Proceedings of International Conference on Telecommunications
    ICT 2000, May 2000.

    Submit questions to:

	Dr. Chung Kei Wong                  Email:  ckwong@alumni.utexas.net
	Inktomi Corporation
	Foster City, CA 94080 USA

        or

        Professor Simon S. Lam              Email:  lam@cs.utexas.edu
        Dept. of Computer Sciences          Phone:  (512) 471-9531
        The University of Texas at Austin   Fax:    (512) 471-8885
        Austin, Texas 78712 USA

 */

#include <netinet/in.h>
#include <sys/time.h>
#include "keygraph.h"

#ifndef commontype
#define commontype
typedef	unsigned char	u8;
typedef unsigned short	u16;
typedef unsigned long	u32;
typedef	signed char	s8;
typedef signed short	s16;
typedef signed long	s32;
#endif


/*************************************************************************
  general constants
 *************************************************************************/

#ifndef FALSE
#define FALSE		0
#endif
#ifndef TRUE
#define TRUE		1
#endif

#define NONE		0
#define RESERVED	0
#define UNKNOWN		0

#define USER_ORIENTED	1
#define KEY_ORIENTED	2
#define GROUP_ORIENTED	3
#define GRAPH_ORIENTED	4

/*************************************************************************
  secure group, key graph, sa, acl
 *************************************************************************/

#define MAX_GROUPS		32	/* max number of secure groups */
#define MAX_GRPNAME		32	/* max length of group name */
#define MAX_TREES		64	/* max number of trees */
#define FIRST_USER_ID		2	/* first user ID */
#define FIRST_AVAIL_SPI		100000	/* first available spi */
#define ADDR_MAX		40
#define MAX_REGISTRARS		16	/* max number of registers */

/* note that SA SPI from 0 to FIRST_AVAIL_SPI-1 are used for individual SA */
/* ie, at most FIRST_AVAIL_SPI users can be supported */

/* group ID is 2^i (i.e., only one 1 in the ID's binary representation) */
/* that is for a 32-bit long ID, there are at most 32 groups */

/*************************************************************************
  user DB
 *************************************************************************/

/* user record */
/* user id is unqiue in a secure group and is given by the server */
typedef struct User_ {
    char *name;				/* user name */
    u32 id;				/* user id given by server */
    SA *indSA;				/* individual key and public key */
    int numTrees;			/* number of trees */
    struct Tree_ *tree[MAX_GROUPS];	/* JL-trees joined */
    u32 cover[MAX_GROUPS];
    KNode * iknode[MAX_GROUPS];		/* individual knode in each tree */
    u32 grpsAllowed;			/* group allowed to join */
    u32 grpsJoined;			/* group joined */
    int sk;				/* tcp socket connection if > 0 */
} User;

typedef struct UserDB_ {
    User **user;			/* pointers to user records */
    int size, max;			/* database size and max size */
} UserDB;


/*************************************************************************
  request
 *************************************************************************/

typedef struct Request_ {
    /* information from the request msg */
    int numGrps;			/* number of secure groups */
    char *grpName[MAX_GROUPS];		/* group names */
    struct Group_ *grp[MAX_GROUPS];	/* group pointers */
    u8 grpStatus[MAX_GROUPS];		/* GRANTED/DENIED and JOIN/LEAVE */
    u32 grpsJoin, grpsLeave, grpsRekey;	/* groups to join, leave, and rekey */
    /* trees (JLs) to join/leave */
    int numTrees;
    struct Tree_ *tree[MAX_GROUPS];	/* JL's to join/leave */
    u8 treeStatus[MAX_GROUPS];		/* JOIN/LEAVE */
    KNodeLst * changeNodes[MAX_GROUPS];	/* knodes to be changed */
					/* for req rekey msg construction */

    char *userName;			/* user name */
    u8 version, type;			/* request msg version and type */
    u32 msgID;				/* msg id */
    /* socket from which this request received */
    struct sockaddr_in userSkAddr;
    int userLen, sk;
    /* others information */
    User *userRec;
} Request;

typedef struct ReqList_ {
    Request *req;
    struct ReqList_ *next;
} ReqList;


/*************************************************************************
  secure group
 *************************************************************************/

typedef struct AddrRec_ {
    char addr[ADDR_MAX];	/* ip address */
    int port, sk;		/* port and socket */
    struct sockaddr_in skAddr;
} AddrRec;

/* tree/JLpattern record */
typedef struct Tree_ {
    char name[MAX_GRPNAME];		/* tree name (a string) */
    u32 cover;				/* cover set */
    int coverSize;			/* size of cover */
    /* pointers to covered base groups */
    int numParents;
    struct Group_ *parent[MAX_GROUPS];
    KTree ktree;			/* key tree (graph) */

    int encryption;			/* encryption algorithm */
    /* store encrypted keys for multiple parents to use */
    u8 store[MAX_MSGSIZE];
    MsgRec rekeyMsg;
} Tree;

/* group record */
typedef struct Group_ {
    char name[MAX_GRPNAME];		/* group name (a string) */
    u32 id, supergrps, subgrps;		/* group id (position of one) */
    /* pointers to immediate supergroups */
    int numParents;
    struct Group_ *parent[MAX_GROUPS];
    /* pointers to trees that (directly) cover this group */
    int numTrees;
    Tree *tree[MAX_GROUPS];
    KNode *knode;			/* group key knode */

    /* rekey and request addresses, ports, and sockets */
    int rekeying;			/* rekeying strategy */
    AddrRec rekeyAddr, reqAddr;
    struct RekeyCtx_ *rekeyCtx;
    int encryption;			/* encryption algorithm */
    int degree;				/* key tree degree */
    u32 rekeySeq;			/* sequence number of rekey msg */
    int joining;			/* joining point algorithm */
    /* store encrypted keys for multiple parents to use */
    u8 store[MAX_MSGSIZE];
    MsgRec rekeyMsg;
} Group;

typedef struct Registrar_ {
    int sk;				/* socket */
    SA *regSA;				/* register SA */
} Registrar;

typedef struct UserSkDB_ {
    int *sk;
    User **user;
    int size, max;
} UserSkDB;

/* server record (for multiple groups) */
typedef struct Server_ {
    int numGrps, maxGrps;		/* number and max # of groups */
    Group *grp[MAX_GROUPS];		/* groups (subgroups) */
    int numTrees, maxTrees;		/* number and max # of trees */
    Tree *tree[MAX_TREES];		/* trees */
    /* same request port for all groups */
    /* same rekey address if graph-oriented rekeying */
    AddrRec rekeyAddr, reqAddr;
    struct RekeyCtx_ *rekeyCtx;
    /* a common user DB is needed to reuse individual keys in diff. groups */
    UserDB *userDB;
    /* for tcp unicast rekey delivery */
    UserSkDB *userSkDB;
    /* the SAs in JLsets belong to multiple groups */
    /* and therefore need to be assigned from a common spi space */
    u32 nextSPI;			/* next SA ID to be assigned */
    u32 nextUserID;			/* next available user ID */
    u32 rekeySeq;			/* sequence number of rekey msg */
    int numReqs;			/* number of requests */
    ReqList reqList;			/* requests (not yet done) */
    /* server SA including signing key, signature scheme, msg digest */
    int msgDigest;			/* message digest function */
    int signature;			/* signature scheme */
    SA *serverSA;			/* server SA */
    /* default for all groups/trees */
    int rekeying;			/* rekeying strategy */
    struct timeval rekeyPeriod;		/* rekey period */
    int encryption;			/* encryption algorithm */
    int degree;				/* key tree degree */
    int joining;			/* joining point algorithm */
    /* registers */
    int numRegistrars;			/* number of registers */
    int maxRegistrars;			/* max number of registers */
    Registrar *regRec[MAX_REGISTRARS];	/* register records */
    AddrRec regReqAddr;
    /* access control */
    char acFile[80];			/* access control filename */
    char cListFile[80];			/* client list (for registrar) */
} Server;

